from nesteddict import set_by_path
import nesteddict
from typing import Optional, Union
from typer import Typer, Argument
from pathlib import Path
import json
import yaml
from enum import Enum
from rich import print
import toml

class SupportedFormats(str, Enum):
	yaml = "yaml"
	toml = "toml"
	json = "json"

CLI = Typer()
readable_file = dict(default="-", allow_dash=True, file_okay=True, resolve_path=True, readable=True, exists=True)

@CLI.command("json")
def as_json(content: Path = Argument(**readable_file), indent: int = 0) -> str:
	return json.dumps(loads(content.read_text()), indent=indent)

@CLI.command("yaml")
def as_yaml(content: Path = Argument(**readable_file)) -> str:
	return yaml.dump(loads(content.read_text()))

@CLI.command("from")
def convert(content: Path = Argument(**readable_file), format: Optional[SupportedFormats] = None) -> str:
	raw = content.read_text()
	module = {
		"yaml": yaml,
		"toml": toml,
		"json": json,
	}[format or detect_format(format)]
	return dumps(module.loads(raw))


def loads(raw: str) -> dict:
	"""
	Loads a tabtree file into a dict of dicts of ... of strings

	>>> loads(
	... '''
	... 1
	... 	1	def lim avec ε=1
	... 	2	dl de sh en x=1/n
	... 2
	... 	1	X=0
	... 	2(mod)
	... 	3.b
	... '''
	... )
	{'1': {'1': 'def lim avec ε=1', '2': 'dl de sh en x=1/n'}, '2': {'1': 'X=0', '2(mod)': None, '3.b': None}}
	"""
	loaded = {}
	loaded_lines = []
	parent_keys = []
	edges = []
	for i, line in enumerate(raw.splitlines()):
		previous_edges = edges
		if line.startswith('#') or not line.strip():
			continue
	
		edges = line.strip().split('\t')
		if not edges:
			continue
		
		# Counts all leading tabs
		depth = 0
		for c in line:
			if c == '\t': depth += 1
			else:
				break
		
		if depth == len(parent_keys) + 1:
			parent_keys.append(previous_edges[-1])
		
		if depth > len(parent_keys):
			raise SyntaxError(f"oopsie @ line {i+1}: some value has no parent while indentation suggests otherwise:\n\t{line}\n\tdepth: {depth}, parent depth: {len(parent_keys)}")
	
		# print(i+1, repr(line).replace(r'\t', '[bold cyan]→[/]'))
		# print("[blue]depth[/]", depth)
		# print("[blue]parents[/]", parent_keys)

		loaded_line = {}
		key_path = parent_keys[:depth] + edges[:-1]
		for i in range(len(key_path)):
			set_by_path(loaded_line, key_path[:i+1], {})
		if not key_path:
			set_by_path(loaded_line, edges[-1], None)
		else:
			set_by_path(loaded_line, key_path, edges[-1])
		# print('[blue]line[/]', loaded_line)
		loaded_lines.append(loaded_line)
		parent_keys = key_path or [edges[-1]]

	"""
	[
		{'1': None},
		{'1': {'1': 'def lim avec ε=1'}},
		{'1': {'2': "dl de sh en x=1/n, dl de tan => 1+tan^2 = tan' -primitive-> tan... pour avoir du o(x^3) et faire la diff en +oo"}},
		{'2': None},
		{'2': {'1': 'X=0 et (×X puis X=-1), donne A=3 et B=4'}},
		{'2': '2(mod)'},
		{'2': {'2(mod)': '4 gprejk'}},
		{'2': '3.b'}
	]

	to 

	{
		'1': {
			'1': 'def lim avec ε=1',
			'2': "dl de sh en x=1/n, dl de tan => 1+tan^2 = tan' -primitive-> tan... pour avoir du o(x^3) et faire la diff en +oo"
		},
		'2': {
			'1': 'X=0 et (×X puis X=-1), donne A=3 et B=4',
			'2(mod)': '4 gprejk',
			'3.b': None
		}
	}
	"""

	loaded = {}
	for loaded_line in loaded_lines:
		print([loaded|loaded_line, loaded])
		print(list(map(nesteddict.depth, [loaded|loaded_line, loaded])))
		if nesteddict.depth(loaded | loaded_line) > nesteddict.depth(loaded):
			loaded |= loaded_line
		else:
			continue
	
	print(loaded)

		

def dumps(o: dict) -> str:
	pass



def detect_format(raw: str) -> SupportedFormats:
	pass

# TODO: def select():

if __name__ == "__main__":
	loads(
	'''
1
	1	def lim avec ε=1
	2	dl de sh en x=1/n, dl de tan => 1+tan^2 = tan' -primitive-> tan... pour avoir du o(x^3) et faire la diff en +oo

2
	1	X=0 et (×X puis X=-1), donne A=3 et B=4
	2(mod)	
		4 gprejk
	3.b	
	'''
	)